#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package ${package}.process;

import java.util.Collections;

import ${package}.data.ProcessData;
import ${package}.event.AbstractEventProcessor;
import ${package}.event.Event;
import ${package}.event.EventResult;
import org.springframework.data.mongodb.core.MongoTemplate;

/**
 * Execute an unfinished process
 * 
 * <h1>Input</h1>
 * 
 * Process name
 * 
 * <h1>Data</h1>
 * 
 * A MongoDB collection containing unprocessed data.
 * 
 * <h1>Actions</h1>
 * 
 * Fetches the process from the MongoDB collection and executes it.
 * There is no real processing for this sample, but the basic principle of
 * running a process from associated data, marking it as done and recording
 * the result is demonstrated.<br/>
 * A certain number of failures are demostrated as well.
 * 
 * <h1>Output</h1>
 * 
 * {@link ${symbol_pound}EVENT_NAME_PROCESS_DONE}: The process name<br/>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ExecuteProcess extends AbstractEventProcessor
{
    public static final String EVENT_NAME_PROCESS_DONE = "processDone";
    
    private final MongoTemplate mongo;
    private final String processData;
    private String eventNameProcessDone;

    /**
     * @param mongo                 connection to MongoDB
     * @param processData           name of DB collection containing process details
     */
    public ExecuteProcess(MongoTemplate mongo, String processData)
    {
        super();
        this.mongo = mongo;
        this.processData = processData;
        this.eventNameProcessDone = EVENT_NAME_PROCESS_DONE;
        
        // Initialize indexes
        ProcessData.checkIndexes(mongo, processData);
    }

    /**
     * Override the {@link ${symbol_pound}EVENT_NAME_PROCESS_DONE default} event name for 'process done'.
     */
    public void setEventNameProcessDone(String eventNameProcessDone)
    {
        this.eventNameProcessDone = eventNameProcessDone;
    }

    @Override
    public EventResult processEvent(Event event) throws Exception
    {
        // Get the user email
        String processName = (String) event.getDataObject();
        
        // Locate the process
        ProcessData process = ProcessData.findProcessByName(mongo, processData, processName);
        // A quick double-check
        EventResult result = null;
        if (process == null)
        {
            result = new EventResult(
                    "Skipping processing for '" + processName + "'.  Process not found.",
                    Collections.<Event>emptyList(),
                    false);
        }
        
        // Simulate some process delay
        synchronized (processName)
        {
            try
            {
                processName.wait((long)(Math.random()* 1000.0) + 10L);
            }
            catch (InterruptedException e)
            {
                // Ignore
            }
        }
        /*
         * Some work
         */
        
        switch ((int)(Math.random() * 10.0))
        {
            // 10% exceptions
            case 0:
                // We let the framework handle this
                throw new RuntimeException("Process " + processName + " failed.");
            // 20% some error condition
            case 1:
            case 2:
                result = new EventResult(
                        "This can be any serializable data e.g. captured Http response.",
                        Collections.<Event>emptyList(),
                        false);
                break;
            // 70% success
            default:
                // Update the process to mark it as done
                ProcessData.markAsDone(mongo, processData, processName);
                // Create 'done' event
                Event doneEvent = new Event(eventNameProcessDone, 0L, processName);
                result = new EventResult("Process " + processName + " completed.", doneEvent);
                break;
        }
        // Done
        return result;
    }
}
