#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package ${package}.process;

import java.util.Collections;

import ${package}.data.ProcessData;
import ${package}.event.AbstractEventProcessor;
import ${package}.event.Event;
import ${package}.event.EventResult;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

/**
 * Prepare a specific number of processes.<br/>
 * This merely creates the test-local data that will be used for later processing.
 * 
 * <h1>Input</h1>
 * 
 * No input requirements
 * 
 * <h1>Data</h1>
 * 
 * A MongoDB collection (either empty or with existing process data).
 * 
 * <h1>Actions</h1>
 * 
 * Examines the MongoDB collection and ensures that there are at least a certain number
 * of unfinished processes existing.
 * 
 * <h1>Output</h1>
 * 
 * {@link ${symbol_pound}EVENT_NAME_PROCESSES_CREATED}: No data<br/> 
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class PrepareProcesses extends AbstractEventProcessor
{
    public static final long DEFAULT_USERS_PER_DOMAIN = 100L;
    public static final String EVENT_NAME_PROCESSES_CREATED = "processesCreated";
    
    private final MongoTemplate mongo;
    private final String processData;
    private final long processCount;
    private String eventNameProcessesCreated;

    /**
     * @param mongo                 connection to MongoDB
     * @param processData           name of DB collection containing process data
     * @param processCount          number of unfinished processes to create
     */
    public PrepareProcesses(MongoTemplate mongo, String processData, long processCount)
    {
        super();
        this.mongo = mongo;
        this.processData = processData;
        this.processCount = processCount;
        this.eventNameProcessesCreated = EVENT_NAME_PROCESSES_CREATED;
        
        // Initialize indexes
        ProcessData.checkIndexes(mongo, processData);
    }

    /**
     * Override the {@link ${symbol_pound}EVENT_NAME_PROCESSES_CREATED default} event name when processes have been created.
     */
    public void setEventNameUsersCreated(String eventNameProcessesCreated)
    {
        this.eventNameProcessesCreated = eventNameProcessesCreated;
    }

    @Override
    public EventResult processEvent(Event event) throws Exception
    {
        // Count the number of processes we already have
        Criteria criteriaNotDone = Criteria.where(ProcessData.FIELD_DONE).is(Boolean.FALSE);
        Query queryNotDone = new Query(criteriaNotDone);
        long notDoneCount = mongo.count(queryNotDone, processData);
        
        long toCreate = processCount - notDoneCount;
        
        long count = 0L;
        while (count < toCreate)
        {
            String processName = "PROC-" + System.currentTimeMillis() + "-" + (count);
            ProcessData data = new ProcessData();
            data.setProcessName(processName);
            data.setDone(false);
            // Persist
            try
            {
                mongo.insert(data, processData);
                // Success
            }
            catch (Throwable e)
            {
                // Just ignore it
                continue;
            }
            // Successful insert
            count++;
        }
        
        // We need an event to mark completion
        Object outputEventData = new DataThatWillNotSerialize();        // Use event data that won't persist
        Event outputEvent = new Event(eventNameProcessesCreated, 0L, outputEventData);

        // Create result
        String msg = "Created " + count + " processes.";
        EventResult result = new EventResult(msg, Collections.singletonList(outputEvent));
        return result;
    }
    
    /**
     * A class used to demonstrate how unserializable data is transfered in events
     * 
     * @author derekh
     * @since 1.1
     */
    private static final class DataThatWillNotSerialize
    {
        
    }
}
