#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package ${package}.process;

import java.util.ArrayList;
import java.util.List;

import ${package}.data.ProcessData;
import ${package}.event.AbstractEventProcessor;
import ${package}.event.Event;
import ${package}.event.EventResult;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

/**
 * Schedule processing events.
 * 
 * <h1>Input</h1>
 * 
 * No input requirements
 * 
 * <h1>Data</h1>
 * 
 * A MongoDB collection containing unprocessed data.
 * 
 * <h1>Actions</h1>
 * 
 * The MongoDB collection is examined to find (see {@link ProcessData${symbol_pound}isDone() unfinished} processes.
 * Unfinished processes are scheduled up to a batch size after which this processor reschedules itself.
 * 
 * <h1>Output</h1>
 * 
 * {@link ${symbol_pound}EVENT_NAME_PROCESS}: The process name<br/>
 * {@link ${symbol_pound}EVENT_NAME_SCHEDULE_PROCESSES}: No data<br/> 
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ScheduleProcesses extends AbstractEventProcessor
{
    public static final int DEFAULT_BATCH_SIZE = 100;
    public static final String EVENT_NAME_PROCESS = "process";
    public static final String EVENT_NAME_SCHEDULE_PROCESSES = "scheduleProcesses";
    
    private final MongoTemplate mongo;
    private final String processData;
    private final long timeBetweenProcesses;
    private String eventNameProcess;
    private String eventNameScheduleProcesses;

    /**
     * 
     * @param mongo                 connection to MongoDB
     * @param processData           name of DB collection containing process details
     * @param processCount          total number of processes to schedule
     * @param timeBetweenProcesses  how long between each process
     */
    public ScheduleProcesses(MongoTemplate mongo, String processData, long timeBetweenProcesses)
    {
        super();
        this.mongo = mongo;
        this.processData = processData;
        this.timeBetweenProcesses = timeBetweenProcesses;
        this.eventNameProcess = EVENT_NAME_PROCESS;
        this.eventNameScheduleProcesses = EVENT_NAME_SCHEDULE_PROCESSES;
        
        // Initialize indexes
        ProcessData.checkIndexes(mongo, processData);
    }

    /**
     * Override the {@link ${symbol_pound}EVENT_NAME_PROCESS default} event name to process an event
     */
    public void setEventNameProcess(String eventNameProcess)
    {
        this.eventNameProcess = eventNameProcess;
    }

    @Override
    public EventResult processEvent(Event event) throws Exception
    {
        int batchSize = 100;    // Hard coded but can be parameterized
        
        // Find out how many registrations there are
        Criteria criteriaNotDone = Criteria.where(ProcessData.FIELD_DONE).is(Boolean.FALSE);
        Query queryNotDone = new Query(criteriaNotDone).skip(0).limit(batchSize);
        
        // Grab the next batch of unfinished processes
        List<ProcessData> processes = mongo.find(queryNotDone, ProcessData.class, processData);
        
        // Each process needs an event
        List<Event> events = new ArrayList<Event>(batchSize + 1);
        long now = System.currentTimeMillis();
        long scheduled = now;
        int count = 0;
        for (ProcessData process : processes)
        {
            scheduled += timeBetweenProcesses;
            // We will attach process name as the event data
            Event eventOut = new Event(eventNameProcess, scheduled, process.getProcessName());
            events.add(eventOut);
            count++;
        }
        // If we had a full batch, reschedule this processor (for just after the last one)
        if (events.size() >= batchSize)
        {
            Event eventOut = new Event(eventNameScheduleProcesses, now, null);
            events.add(eventOut);
        }
        // Create the result of this processor
        EventResult result = new EventResult(
                "Created " + count + " scheduled processes.",
                events);
        return result;
    }
}
